<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Api;

use MAM\Daemon\Core\Mod;
use MAM\Daemon\Core\Player;
use MAM\Daemon\Core\Registry;
use MAM\Daemon\Libraries\Helper\Helper;
use MAM\Daemon\Libraries\Logging\LogInterface;
use MAM\Daemon\Libraries\Permissions\Permissions;

class Api {
    /**
     * @var ApiInterface
     */
    private $concrete_api;
    /**
     * @var LogInterface
     */
    private $logging;
    /**
     * @var string[]
     */
    private $allowed_api_types = ['http'];
    /**
     * @var array
     */
    private $api_users = [];
    /**
     * @var Mod
     */
    private $mod;
    /**
     * @var Player[]
     */
    private $players;
    /**
     * @var Permissions
     */
    private $permissions;

    public function __construct($api_type, $port) {
        $this->logging = Registry::getInstance()->logging;
        $this->mod = Registry::getInstance()->mod;
        $this->players = &Registry::getInstance()->players;
        $this->permissions = Registry::getInstance()->permissions;

        if (!in_array($api_type, $this->allowed_api_types, true)) {
            $error_message = 'API: invalid api type (' . $api_type . '), only the following types are supported: ' . implode(', ', $this->allowed_api_types);
            $this->logging->warning($error_message);
            throw new \Exception($error_message);
        }

        switch ($api_type) {
            case 'http':
                $this->concrete_api = new HttpServer($port);
                break;
            default:
                $error_message = 'API: not yet suppported api type: ' . $api_type;
                $this->logging->warning($error_message);
                throw new \Exception($error_message);
        }

        /* @var $jobs \MAM\Daemon\Libraries\Jobs\Jobs */
        $jobs = Registry::getInstance()->jobs;
        $jobs->addEveryTimeJob(array($this, 'doWork'));

        $this->api_users = Helper::parseJson($this->mod->getConfigDir() . '/apiusers.json', false);
    }

    public function doWork() {
        $this->concrete_api->doWork();

        foreach ($this->concrete_api->getClients() as $api_client) {
            if ($api_client->readRequest()) {
                $user = $api_client->getUser();
                if ($user === false || !isset($this->api_users[$user]) || $this->api_users[$user]['sha512password'] !== hash('sha512', $api_client->getPassword())) {
                    $api_client->requestAuth();
                } else {
                    $api_client->executeRequest($this, $this->api_users[$user]['group']);
                }
                $api_client->disconnectClient();
            }
        }
    }

    public function getPlayers($group) {
        if (!$this->permissions->canExecuteCommand($group, 'api.getplayers')) {
            return false;
        }
        $ret = [];
        foreach ($this->players as $player) {
            $ret[] = $player->getPlayerInfos();
        }
        return $ret;
    }

    public function getConfig($group, $file) {
        if (!$this->permissions->canExecuteCommand($group, 'api.getconfig')) {
            return false;
        }
        $safe_file_path = $this->mod->getConfigDir() . '/' . Helper::makeSafeFilePath($file);
        if (!is_readable($safe_file_path)) {
            return false;
        }
        return file_get_contents($safe_file_path);
    }

    public function writeConfig($group, $file, $content) {
        if (!$this->permissions->canExecuteCommand($group, 'api.writeconfig')) {
            return false;
        }
        $safe_file_path = $this->mod->getConfigDir() . '/' . Helper::makeSafeFilePath($file);
        if (!is_writable($safe_file_path)) {
            return false;
        }
        return file_put_contents($safe_file_path, $content) !== false;
    }

    public function deleteUser($group, $guid) {
        if (!$this->permissions->canExecuteCommand($group, 'api.deleteuser')) {
            return false;
        }
        //@todo implement
    }

    public function addUser($group, $user_json) {
        if (!$this->permissions->canExecuteCommand($group, 'api.adduser')) {
            return false;
        }
        //@todo implement
    }

    public function newToken($group, $token_group) {
        if (!$this->permissions->canExecuteCommand($group, 'api.newtoken')) {
            return false;
        }
        //@todo implement
    }

    public function getTokens($group) {
        if (!$this->permissions->canExecuteCommand($group, 'api.gettokens')) {
            return false;
        }
        //@todo implement
    }

    public function deleteToken($group, $token) {
        if (!$this->permissions->canExecuteCommand($group, 'api.deletetoken')) {
            return false;
        }
        //@todo implement
    }

    public function executeIngameCommand($group, $command_string) {
        if (!$this->permissions->canExecuteCommand($group, 'api.executeingamecommand')) {
            return false;
        }
        //@todo implement
    }
} 