<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Core;

use \MAM\Daemon\Libraries\Weapons\Weapons;
use \MAM\Daemon\Libraries\Helper\Helper;

/**
 * Contains all main functions
 *
 * This class contains all main functions of the adminmod
 *
 * @author mirko911
 */
class Mod {
    /**
     * Path of the config dir
     *
     * @var string
     */
    protected $config_dir = false;
    /**
     * Storage of config variables
     * @var array
     */
    protected $config = array();
    /**
     * Storage of Maps
     *
     * @var array
     */
    protected $maps = array();
    /**
     * Storage of Gametypes
     *
     * @var array
     */
    protected $gametypes = array();
    /**
     * Weapon Class
     *
     * Handles weapons and extensions
     *
     * @var \MAM\Daemon\Libraries\Weapons\Weapons
     */
    protected $weapons = array();
    /**
     * Storage of groups
     *
     * @var array
     */
    protected $groups = array();
    /**
     * Storage of admins
     *
     * @var array
     */
    protected $admins = array();
    /**
     * Storage of reasons
     * @var array
     */
    protected $reasons = array();
    /**
     * Storage of Events for registerEvent
     *
     * @var array
     */
    protected $plugin_events = array();
    /**
     * Storage of Hooks for registerHook
     *
     * @var array
     */
    protected $plugin_hooks = array();
    /**
     * Default Cvars for Plugins
     *
     * @var array
     */
    protected $default_cvars = array();
    /**
     * Stores the latest gametype
     *
     * @var string
     */
    protected $gametype = false;
    /**
     * Storage for commands
     *
     * @var array
     */
    protected $commands = array();
    /**
     * Storage of help language strings
     *
     * @var array
     */
    protected $help = array();
    /**
     * Storage of language strings
     *
     * @var array
     */
    protected $language = array();
    /**
     * Storage of the latest maps
     *
     * @var array
     */
    public $lastmaps = array();
    /**
     * The game from stratparameter
     *
     * @var string
     */
    protected $game = false;
    /**
     * Config file names
     *
     * @var array
     */
    protected $configfiles = array(
        'admins' => 'users.json',
        'groups' => 'groups.json',
        'config' => 'config.json',
        'reasons' => 'reasons.json',
        'aliases' => 'aliases.json'
    );
    /**
     * Name of the Engine
     *
     * @var string
     */
    protected $engine = '';
    /**
     * Class for Remote Commands
     *
     * @var \MAM\Daemon\Engine\Quake3\Rcon\RconCommands
     * @var \MAM\Daemon\Engine\Frostbite3\Rcon\RconCommands
     */
    public $rcon;
    /**
     * Class for player related stuff
     *
     * @var \MAM\Daemon\Core\Player
     */
    protected $players;
    /**
     * Class for Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    protected $logging;

    /**
     * Constructor, sets the references of logging, players and rcon.
     */
    public function __construct() {
        $registry = Registry::getInstance();
        $this->logging = $registry->logging;
        $this->players = & $registry->players;
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return 'mod';
    }

    /**
     * Sets the filename of one configuration file (without path).
     * @example setConfigFileName("config", "config.cfg")
     * @param   string      $config     kind of configuration file
     * @param   string      $filename   filename (without path)
     */
    public function setConfigFileName($config, $filename) {
        $this->configfiles[$config] = $filename;
    }

    /**
     * Sets the path to the configuration files.
     * @param   string      $dir        Path to config dir [default configs]
     * @return  bool        true if setting was successful, false if string $dir is no valid directory
     */
    public function setConfigDir($dir) {
        if (is_dir($dir)) {
            $this->config_dir = $dir;
            return true;
        }
        return false;
    }

    /**
     * Reads all the configuration files and writes the content into the arrays:
     *      admins, config, groups, maps and reasons
     * Furthermore the language files and the names of the weapons will be parsed.
     *
     */
    public function readConfig() {
        $this->admins = array();
        $this->config = array();
        $this->groups = array();
        $this->maps = array();
        $this->reasons = array();

        //Check required files
        foreach ($this->configfiles as $file) {
            if (!is_readable($this->config_dir . DIRECTORY_SEPARATOR . $file)) {
                $this->logging->error("Can't open config file '$this->config_dir/$file'");
            }
        }

        //Config.json and plugins directory
        $config = Helper::parseJson($this->config_dir . DIRECTORY_SEPARATOR . $this->configfiles["config"]);
        $this->logging->info("Config loaded: " . $this->configfiles["config"]);
        $dir = opendir($this->config_dir . "/plugins");
        while (($file = readdir($dir)) !== false) {
            if (substr($file, -5) !== ".json") {
                continue;
            }
            // the cast to array just makes sure that null wont get passed (needed since json_decode("", true) returns null)
            $tmp = (array) Helper::parseJson($this->config_dir . "/plugins/$file");
            $config = Helper::arrayMergeRecursiveDistinct($config, $tmp);
            unset($tmp);
            $this->logging->info("Config loaded: plugins/$file");
        }
        $this->config = $config;

        //maps.json
        $maps = Helper::parseJson('daemon' . DIRECTORY_SEPARATOR . 'engine' . DIRECTORY_SEPARATOR . $this->getEngine() . DIRECTORY_SEPARATOR . $this->getGame() . DIRECTORY_SEPARATOR . 'data' . DIRECTORY_SEPARATOR . 'maps.json');
        if (is_file($this->config_dir . DIRECTORY_SEPARATOR . 'maps.json')) {
            $tmp = (array) Helper::parseJson($this->config_dir . DIRECTORY_SEPARATOR . 'maps.json');
            $maps = Helper::arrayMergeRecursiveDistinct($maps, $tmp);
            $this->logging->info('Config loaded: maps.json');
        }
        $this->maps = $maps;


        //gametypes.json
        $gametypes = Helper::parseJson('daemon' . DIRECTORY_SEPARATOR . 'engine' . DIRECTORY_SEPARATOR . $this->getEngine() . DIRECTORY_SEPARATOR . $this->getGame() . DIRECTORY_SEPARATOR . 'data' . DIRECTORY_SEPARATOR . 'gametypes.json');
        if (is_file($this->config_dir . DIRECTORY_SEPARATOR . 'gametypes.json')) {
            $tmp = (array) Helper::parseJson($this->config_dir . DIRECTORY_SEPARATOR . 'gametypes.json');
            $maps = Helper::arrayMergeRecursiveDistinct($gametypes, $tmp);
            $this->logging->info('Config loaded: gametypes.json');
        }
        $this->maps = $maps;

        //users.json
        $this->admins = Helper::parseJson($this->config_dir . DIRECTORY_SEPARATOR . $this->configfiles['admins']);
        foreach ($this->admins as $key => $value) {
            if (!is_array($value['names'])) {
                $this->admins[$key]['names'] = array();
                continue;
            }
            $this->admins[$key]['names'] = array_map('strtolower', $value['names']);
        }
        $this->logging->info('Config loaded: ' . $this->configfiles['admins']);

        //groups.json
        $this->groups = Helper::parseJson($this->config_dir . DIRECTORY_SEPARATOR . $this->configfiles["groups"]);
        $this->logging->info("Config loaded: " . $this->configfiles["groups"]);

        //reasons.json
        $this->reasons = Helper::parseJson($this->config_dir . DIRECTORY_SEPARATOR . $this->configfiles["reasons"])['reasons'];
        $this->logging->info("Config loaded: " . $this->configfiles["reasons"]);

        // aliases.json einlesen
        $this->config['aliases'] = Helper::parseJson($this->config_dir . DIRECTORY_SEPARATOR . $this->configfiles['aliases'])['aliases'];
        $this->logging->info('Config loaded: ' . $this->configfiles['aliases']);

        //languages
        $this->readLanguageFile($this->getCV("daemon", "language"));

        //Waffen einlesen
        // $weaponparser =  new Daemon\Libraries\Weaponparser\Weaponparser();
        // $this->weapons = $weaponparser->parse();
        //set the timezone
        date_default_timezone_set($this->getCV("daemon", "timezone"));

        $this->triggerEvent("parseConfig");
        $this->logging->info("All config files were parsed");
    }

    /**
     * Reads extra config file and merge that with current config. Map specific
     * configurations or individual settings can be load with this function.
     * @param   string      $file       filename of the config file
     * @return  bool        true if successful
     */
    public function readConfigFile($file) {
        $config = $this->config_dir . "/$file";
        if (!is_readable($config)) {
            $this->logging->warning("Can't open config file '$file'");
            return false;
        }

        $cfg = (array) Helper::parseJson($config);
        $this->config = Helper::arrayMergeRecursiveDistinct($this->config, $cfg);
        $this->logging->info("Config loaded: $file");


        $this->triggerEvent("parseConfig");

        return true;
    }

    /**
     * Reads the language and help files and saves them into $this->langauge and $this->help.
     * @param   string      $dir        language code, e.g. "de" or "en", represents the subdirectory in the languages folder
     */
    protected function readLanguageFile($dir) {
        $this->help = array();
        $this->language = array();

        $data = Helper::parseJson("languages/$dir/daemon.json");

        $this->language = $data['mam.daemon'];

        $this->help = $data['mam.daemon.help'];
    }

    /**
     * Returns the language string and replace the search array/string with the replace array/string.
     * See php's documentation for str_replace().
     * @param   string      $str        The key of the language string.
     * @param   string[]    $search     Optional Parameters for search and replace (comma separated)
     * @param   string[]    $replace    Optional Parameters for search and replace (comma separated)
     * @return  string      The language string
     */
    public function getLngString($str, $search = array(), $replace = array()) {
        $str = strtolower($str);
        if (!array_key_exists($str, $this->language)) {
            $this->logging->notice("Missing translation for '$str' for language '" . $this->getCV("daemon", "language") . "'");
            return "^1$str";
        }
        return str_replace($search, $replace, $this->language[$str]);
    }

    /**
     * Function calls all user functions which were registered by $this->registerEvent() for one event type.
     * @param   string      $event      Type/name of the event.
     * @param   mixed       $param      Parameters
     * @return  boolean     False if such an event does not exist.
     */
    public function triggerEvent($event, $param = NULL) {
        if (!array_key_exists($event, $this->plugin_events)) {
            return false;
        }
        foreach ($this->plugin_events[$event] as $value) {
            // if($event != "everyTime" && $event != "pluginAdded" && $event != "rconSay" && $event != "mamStarted")
            if (is_object($value["object"])) {
                $value["object"]->$value["function"]($param);
            } elseif (is_string($value["object"])) {
                call_user_func($value["object"] . "::" . $value["function"], $param);
            } else {
                $this->logging->error("Can't trigger function {$value["function"]}, (" . implode(" ", $param) . ") because this function doesn't exists");
            }
        }
        return true;
    }

    /**
     * Registers a function for an event which will be called, if the event occurs.
     * The function can be either a function in global space or a function within an object.
     * @param   string      $event      The event type for the function.
     * @param   string      $function   The name of the function.
     * @param   bool|object $object     The object in which the function exists.
     * @param   bool        $position   Optional position. With $position it's possible to call a special event at first
     */
    public function registerEvent($event, $function, $object, $position = false) {
        $array = array(
            "function" => $function,
            "object" => $object,
        );
        if (!$position) {
            $this->plugin_events[$event][] = $array;
        } else {
            array_splice($this->plugin_events[$event], 3, 0, $array);
        }
    }

    /**
     * Registers a function or closure for a hook which will be called at the hook position.
     * The function can be either a function in global space, function within an object or a closure.
     * @param  string       $hook The hook position
     * @param  callable     $function The function to call
     * @param   bool|object $object $object   The object, Class in which the function exists
     * @return void
     */
    public function registerHook($hook, $function, $object = false) {
        if ($object) {
            $this->plugin_hooks[$hook][] = [$object, $function];
        } else {
            $this->plugin_hooks[$hook][] = $function;
        }
    }

    /**
     * Registers a new mod command.
     * @param   string      $name       The name of the command.
     * @param   string      $syntax     The syntax of the command.
     * @param   string      $function   The name of the function.
     * @param   object      $object     The object in which the function exists.
     */
    public function registerCommand($name, $syntax, $function, &$object) {
        if (!is_object($object)) {
            $this->logging->error('Command ' . $name . ' has no callable object');
        }

        $this->commands[$name] = array(
            "syntax" => $syntax,
            "function" => $function,
            "object" => &$object
        );
    }

    /**
     * Sets a default setting for a configuration entry.
     * @param   string      $section        Section
     * @param   string      $cv             Key
     * @param   mixed       $value          Value
     */
    public function setDefaultCV($section, $cv, $value) {
        $section = mb_strtolower($section);
        $cv = mb_strtolower($cv);
        $this->default_cvars[$section][$cv] = $value;
    }

    /**
     * Returns the value for a CV.
     * If the given CV is not available the function will try to return the default CV (set by $this>setDefaultCV()).
     * If the given CV is neither set nor default available, a MOD_ERROR log entry appears.
     * @param   string      $section        Section
     * @param   string      $cv             Key
     * @return  mixed
     */
    public function getCV($section, $cv) {
        $section = mb_strtolower($section);
        $cv = mb_strtolower($cv);
        if (isset($this->config[$section][$cv])) {
            return $this->config[$section][$cv];
        }
        if (isset($this->default_cvars[$section][$cv])) {
            $this->logging->notice("ConfigVar [$section]$cv NOT set, using default: '" . $this->default_cvars[$section][$cv] . "'");
            //Do not show the Error-Msg multiple times
            $this->config[$section][$cv] = $this->default_cvars[$section][$cv];
            return $this->default_cvars[$section][$cv];
        }
        $this->logging->warning("ConfigVar [$section]$cv NOT set and can't use a default value");
    }

    /**
     * Removes the color codes from the given string.
     * @param   string      $name       String containing color codes (^1 ...)
     * @return  string      text-only
     */
    public function removecolor($name) {
        return preg_replace('|\^.|', "", $name);
    }

    /**
     * Search for an admin in $this->admins[$guid] and returns the admin.
     * If no admin will be found for the given guid, false will be returned.
     * @param string $guid Admins guid
     * @return array|boolean Array with the admins properties (according to admin config) or false if no admin was found for the given guid.
     */
    public function getAdminByGuid($guid) {
        if (array_key_exists($guid, $this->admins)) {
            return $this->admins[$guid];
        }
        return false;
    }

    /**
     * Returns all admins who are online atm
     *
     * Returns a comma seperated list of admin guids who are online atm.
     * @return string[] Comma seperated list of admins who are online
     */
    public function getOnlineAdmins() {
        $online = array();

        foreach (array_keys($this->players) as $check_guid) {
            if (array_key_exists($check_guid, $this->players)) {
                $group = $this->players[$check_guid]->getGroup();
                $group_full = $this->getGroup($group);
                if (!array_key_exists('shownonline', $group_full) || !$group_full['shownonline']) {
                    continue;
                }
                $online[] = $check_guid;
            }
        }

        return  $online;
    }

    /**
     * Checks if the given CV is set in the config.
     * @param string $section Section
     * @param string $cv Key
     * @return bool
     */
    public function existsCV($section, $cv) {
        $section = strtolower($section);
        $cv = strtolower($cv);
        return isset($this->config[$section][$cv]);
    }

    /**
     * Returns the full/long name of the given group
     *
     * @param string $short Shortname of the group
     * @return string
     */
    public function getLongGroupName($short) {
        if (!array_key_exists($short, $this->groups)) {
            return false;
        }
        return $this->groups[$short]["displayname"];
    }

    /**
     * Returns an array of strings with all the commands the group can execute.
     *
     * @param string $group Shortname of the group
     * @return array
     */
    public function getGroupCommands($group) {

        if ($group == "*") {
            return array_keys($this->commands);
        }
        if (!array_key_exists($group, $this->groups)) {
            return array();
        }

        $commands = array();
        foreach ($this->commands as $command_name => $command_params) {
            $namespace = $command_params['object'];
            $permission = $namespace . '.' . $command_name . '.canexecute';
            /** @var \MAM\Daemon\Libraries\Permissions\Permissions Permissions System */
            $permissions = Registry::getInstance()->permissions;
            if ($permissions->canExecuteCommand($group, $permission)) {
                $commands[] = $command_name;
            }
        }

        return $commands;
    }

    /**
     * Returns the guid of a player.
     *
     * @param int|string $search If numeric, the function assumes, that the parameter is the PID.
     * If nothing could be found or if $search is not numeric the function looks for a player with $search in the nickname.
     * @return string|boolean Player's guid or false if no player or more than one player will be found.
     */
    public function findPlayerGuid($search) {
        $find = false;
        if (is_numeric($search)) {
            $find = $this->getPlayerGuidByPID($search);
        }
        if ($find !== false) {
            return $find;
        }
        return $this->findPlayerGuidByName($search);
    }

    /**
     * Returns the guid for the given player id.
     * @param int $pid Player ID
     * @return string|boolean Player's guid or false if no player with this $pid exists.
     */
    public function getPlayerGuidByPID($pid) {
        foreach (array_keys($this->players) as $guid) {
            if ($this->players[$guid]->getPID() == $pid) {
                return $guid;
            }
        }
        return false;
    }

    /**
     * Finds a player with $name in the nickname. If that could be for more than one player, the function will return false.
     * @param string $name
     * @return string|boolean Player's guid or false if not exactly one player will be found.
     */
    public function findPlayerGuidByName($name) {
        $found = false;
        foreach (array_keys($this->players) as $guid) {
            if (stripos($this->players[$guid]->getName(), $name) !== false) {
                if ($found != false) {
                    return false;
                }
                $found = $guid;
            }
        }
        return $found;
    }

    /**
     * Returns the name of the map for the given search string.
     * @param string $search Name of the map (with or without leading "mp_" or longname, see maps.cfg)
     * @return string|boolean Name of the map with leading "mp_" or false if map couldn't be found.
     */
    public function findMap($search) {
        $search = strtolower($search);
        if (array_key_exists($search, $this->maps)) {
            return $search;
        }
        foreach ($this->maps as $short => $long) {
            if (strtolower($long) == $search) {
                return $short;
            }
        }
        if (array_key_exists("mp_" . $search, $this->maps)) {
            return "mp_$search";
        }
        return false;
    }

    /**
     * Returns current gametype.
     * @return string Gametype
     */
    public function getCurrentGametype() {
        return $this->gametype;
    }

    /**
     * Returns the long map name (see maps.cfg) or the short one if it could not be found.
     * @param string $short Mapname with the leading "mp_"
     * @return string Long map name or short if it couldn't be found
     */
    public function getLongMapName($short) {
        $short = strtolower($short);
        if (!array_key_exists($short, $this->maps)) {
            return $short;
        }
        return $this->maps[$short];
    }

    /**
     * Returns all available maps from $this->maps.
     * @param boolean|string $glue If false, the function returns an array with the short mapnames. Otherwise a string with $glue as divisor is returned.
     * @return array|string
     */
    public function getMaps($glue = false) {
        if ($glue) {
            return implode($glue, array_keys($this->maps));
        }
        return array_keys($this->maps);
    }

    /**
     * Returns all available gametypes from $this->gametypes.
     * @param boolean|string $glue If false, the function returns an array with the short gametype. Otherwise a string with $glue as dividor is returned.
     * @return array|string
     */
    public function getGametypes($glue = false) {
        if ($glue) {
            return implode($glue, array_keys($this->gametypes));
        }
        return array_keys($this->gametypes);
    }

    /**
     * Kills the mod (quit daemon).
     */
    public function killMod() {
        $this->rcon->rconSay($this->getLngString("killmod"));
        $this->logging->error("Mod was shut down by user");
    }

    /**
     * Returns configuration directory.
     * @return string configuration directory
     */
    public function getConfigDir() {
        return $this->config_dir;
    }

    /**
     * Returns the gametype
     * @return String
     */
    public function getGameType() {
        return $this->gametype;
    }

    /**
     * Returns an array with the PIDs as keys and the names as values.
     * @return array
     */
    public function getPlayerList() {
        $players = array_keys($this->players);
        $pl = array();
        foreach ($players as $value) {
            $pl[$this->players[$value]->getPID()] = $this->players[$value]->getName();
        }
        return $pl;
    }

    /**
     * Return an array with the $count last maps.
     * @param int $count
     * @return array Filled with mapnames or false if $count maps is too high
     */
    public function getLastMaps($count) {
        $return = array();
        for ($i = 0; $i < $count; $i++) {
            if (array_key_exists(count($this->lastmaps) - $i - 1, $this->lastmaps)) {
                $return[] = $this->lastmaps[count($this->lastmaps) - $i - 1];
            } else {
                $return[] = false;
            }
        }
        return $return;
    }

    /**
     * Writes the admins to the configuration file (users.json).
     */
    public function writeAdminsCfg() {
        file_put_contents($this->config_dir . '/users.json', json_encode($this->admins, JSON_PRETTY_PRINT | JSON_UNESCAPED_SLASHES | JSON_UNESCAPED_UNICODE));
        $this->logging->info('users.json was updated and written to hard disk');
    }

    /**
     * Update an admin's setting.
     * @param string $guid admins guid
     * @param array $update associative array with $key=>$value for the new setting (will overwrite $this->admins[$guid][$key] with $value)
     * @return boolean
     */
    public function updateAdmin($guid, $update) {
        if (!array_key_exists($guid, $this->admins)) {
            return false;
        }
        foreach ($this->admins[$guid] as $key => $value) {
            if (isset($update[$key])) {
                $this->admins[$guid][$key] = $update[$key];
            }
        }
        $uptstr = "";
        foreach ($update as $key => $value) {
            $uptstr .= "$key=>$value; ";
        }
        $this->logging->info("Admin '$guid' was updated: $uptstr");
        $this->writeAdminsCfg();
        return true;
    }

    /**
     * Deletes the admin and writes the configuration file ($this->writeAdminsCfg()).
     *
     * @param String $guid Guid of executing player
     */
    public function deleteAdmin($guid) {
        if (array_key_exists($guid, $this->admins)) {
            unset($this->admins[$guid]);
            $this->writeAdminsCfg();
        }
    }

    /**
     * Adds an admin and writes the configuration files ($this->writeAdminsCfg()).
     * @param string $guid
     * @param string $group
     * @param array $names All the admins names.
     * @param boolean $protected
     * @return boolean false if admin already exists
     * @todo remove $protected
     */
    public function addAdmin($guid, $group, $names = array(), $protected = false) {
        if (array_key_exists($guid, $this->admins)) {
            return false;
        }
        $this->admins[$guid] = array(
            "group" => $group,
            "names" => $names
        );
        if (isset($this->plugin_events['adminAdded'])) {
            $this->triggerEvent('adminAdded', [$guid, &$this->admins]);
        }
        $this->writeAdminsCfg();
        return true;
    }

    /**
     * Returns all key value pairs (associative array) for the given section.
     * If $except parameter is given, that keys are not in the return array.
     * @param string $section Section
     * @param array $except Array with the keys as content that should not be in the return array
     * @return array Associative array with all the section's keys and values (except those in $except)
     */
    public function getCVSection($section, $except = array()) {
        $section = mb_strtolower($section);
        if (empty($this->config[$section])) {
            return array();
        }
        $return = $this->config[$section];
        foreach ($except as $value) {
            if (isset($return[$value])) {
                unset($return[$value]);
            }
        }
        return $return;
    }

    /**
     * Returns an array with $shortname=>$longname.
     * @return array
     */
    public function getGroups() {
        return $this->groups;
    }

    /**
     * Checks if the player (guid) is allowed to have that name.
     *
     * @param String $guid Guid of executing player
     * @param String $name Player Name
     * @return boolean false if this name is protected by an admin, otherwise true
     */
    public function guidCanHaveName($guid, $name) {
        $name = strtolower($name);
        foreach ($this->admins as $key => $value) {
            if (in_array($name, $value['names']) && strtolower($key) != strtolower($guid)) {
                return false;
            }
        }
        return true;
    }

    /**
     * Sets the Game
     * @param String $game Shortname of the game
     */
    public function setGame($game) {
        $this->game = $game;
    }

    /**
     * Returns the game
     * @return string shortname of the game
     */
    public function getGame() {
        return $this->game;
    }

    /**
     * Get the help string of a command
     * @param String $str
     * @return String
     */
    public function getHelpString($str) {
        return $this->help[$str];
    }

    /**
     * Returns and Array of Admins
     * @return array  Key = GUID Value = group, names. If there are no admins, the array will be empty
     */
    public function getAdmins() {
        return $this->admins;
    }

    /**
     * Sets the Admin array and write to users.json
     * @param array   $admins the new admin array
     * @param boolean $force  If set to true it will allow an empty array
     */
    public function setAdmins(array $admins, $force = false) {
        if (!empty($admins) || $force) {
            $this->admins = $admins;
            $this->writeAdminsCfg();
        } else {
            $this->logging->warning('Prohibited deleting Admins. Use setAdmins(Array $admins, true) to force overwrite.');
        }
    }

    /**
     * Returns the group
     *
     * @param String $name The shortname of the group
     * @return array of the group | false when no group was found
     */
    public function getGroup($name) {
        if (!array_key_exists($name, $this->groups)) {
            return false;
        }
        return $this->groups[$name];
    }

    /**
     * Find the reason from reasons.cfg or displays reason
     * Call-by-reference!
     *
     * @param string $reason
     */
    public function findReason(&$reason) {
        if (empty($reason)) {
            return;
        }
        if (array_key_exists(strtolower($reason), $this->reasons)) {
            $reason = $this->reasons[strtolower($reason)];
        }
    }
    //Command Execution

    /**
     * Executes a command.
     * @param string $msg Chatmessage (with leading command prefix)
     * @param string $guid
     * @param boolean $executed (call-by-reference)
     * @return boolean false if execution fails (various reasons)
     */
    public function executeCommand($msg, $guid, &$executed = false) {
        //remove prefix
        $msg = substr($msg, 1);

        //Parameter-Aufbereitung
        $parameters = explode(" ", $msg);

        //Command Aliases
        if ($this->existsCV("aliases", strtolower($parameters[0]))) {
            $msg = substr($msg, strlen($parameters[0]));
            $msg = $this->getCV("aliases", strtolower($parameters[0])) . $msg;
            $parameters = explode(" ", $msg);
        }

        //Eigentlichen Command "herausfiltern"
        $command = strtolower(array_shift($parameters));

        //Existiert der Command?
        if (!array_key_exists($command, $this->commands)) {
            if ($this->getCV("daemon", "wrongcmdmessage")) {
                $this->players[$guid]->say($this->getLngString("unrecognizedCmd", "<COMMAND>", $this->getCV("daemon", "cmdprefix") . $command));
            }
            return false;
        }

        //Darf der Player den command ausf hren?
        if (!$this->players[$guid]->isAllowedToExec($command)) {
            if ($this->getCV("daemon", "wrongcmdmessage")) {
                $this->players[$guid]->say($this->getLngString("forbiddenCmd", "<COMMAND>", $this->getCV("daemon", "cmdprefix") . $command));
            }
            return false;
        }

        // calls all pre hooks and stops execution as soon as the first one fails.
        if (isset($this->plugin_hooks['preCommandExecute'])) {
            foreach ($this->plugin_hooks['preCommandExecute'] as $hook) {
                if (!call_user_func($hook, $guid, $command)) {
                    return false;
                }
            }
        }

        $executed = true;

        //Mindestzeit zwischen den Commands unterschritten?
        if (time() - $this->players[$guid]->lastCommand <= $this->getCV("daemon", "antispam")) {
            $this->logging->info("Command '$command' by player '" . $this->players[$guid]->getName() . "' ignored, spamming, PID: " . $this->players[$guid]->getPID() . ", GUID: " . $guid);
            return false;
        }
        $this->players[$guid]->lastCommand = time();

        //Schreibe Logzeile
        $this->logging->info("Player '" . $this->players[$guid]->getName() . "' executed command: '" . $msg . "', PID: " . $this->players[$guid]->getPID() . ", GUID: " . $guid);

        $cmd = $this->commands[$command];

        //Richtige Syntax?
        if ($cmd["syntax"] != false && !preg_match($cmd["syntax"], $msg)) {
            //Zeige Hilfe
            $this->printHelp($command, $guid);
            return false;
        }

        //F hre die Command-Funktion aus
        if (is_object($cmd["object"])) {
            $cmd["object"]->$cmd["function"]($guid, $parameters);
        } elseif (is_string($cmd["object"])) {
            call_user_func($cmd["object"] . "::" . $cmd["function"], $guid, $parameters);
        } else {
            $cmd["function"]($guid, $parameters);
        }


        //Löse ein Event aus
        $this->triggerEvent("playerExecutedCommand", array($guid, $command, $parameters));
    }

    /**
     * Sends the help message of a command to a specific player.
     * @param string $command
     * @param string $guid
     */
    public function printHelp($command, $guid) {
        if (array_key_exists($command, $this->help)) {
            $this->players[$guid]->say($this->getLngString("helpUsage") . $this->getCV("daemon", "cmdprefix") . $this->help[$command]);
        } else {
            $this->logging->notice("Help for command '$command' wasn't found for language '" . $this->getCV("daemon", "language") . "'");
            $this->players[$guid]->say($this->getLngString("helpNotFound", array("<PREFIX>", "<COMMAND>"), array($this->getCV("daemon", "cmdprefix"), $command)));
        }
    }

    /**
     * Sets the engine
     * @param String $name Name of the Engine (quake3 / frostbite)
     */
    public function setEngine($name) {
        $this->engine = $name;
    }

    /**
     * Returns the engine
     * @return String Name of the Engine
     */
    public function getEngine() {
        return $this->engine;
    }

    /**
     * Returns the long weapon name.
     * @param string $weapon
     * @return string
     */
    public function getWeaponInfo($weapon) {
        if (count($this->weapons) === 0) {
            $this->weapons = new Weapons();
        }
        $weapon_info = $this->weapons->getWeapon($weapon);
        return $weapon_info;
    }
}
