<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Engine\Frostbite3\Rcon;

use MAM\Daemon\Engine\Frostbite3\Maps;
use MAM\Daemon\Core\Registry;

/**
 * Rcon Commands Class for Frostbite Games
 *
 * With this class you can send rcon commands to a bf server
 *
 * @author Master of Little <masteroflittle@manuadminmod.de>
 */
class RconCommands {
    /**
     * Mod
     *
     * @var \MAM\Daemon\Core\Mod
     */
    protected $mod;
    /**
     * Rcon
     *
     * @var \MAM\Daemon\Engine\Frostbite3\Rcon\Rcon
     */
    protected $rcon;
    /**
     * Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    protected $logging;
    /**
     * Players
     *
     * @var \MAM\Daemon\Core\Player
     */
    protected $players;
    /**
     * Maps
     *
     * @var \MAM\Daemon\Engine\Frostbite3\Maps
     */
    protected $maps;
    /**
     * Tools
     *
     * @var \MAM\Daemon\Engine\Frostbite3\Tools
     */
    public $tools;

    /**
     * The constructor of the class
     *
     * @return \MAM\Daemon\Engine\Frostbite3\Rcon\RconCommands
     */
    public function __construct() {

        $registry = Registry::getInstance();
        $this->mod = $registry->mod;
        $this->players = &$registry->players;
        $this->logging = $registry->logging;

        $this->rcon = new Rcon($this->mod->getCV("connection", "ip"), $this->mod->getCV("connection", "port"));

        $this->rcon->setRconpassword($this->mod->getCV("connection", "password"));
        $this->rcon->login();
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __toString function
     * to get it working with the permissions system. Therefore the return of __toString must me the
     * namespace like daemon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return "Frostbite Rconcommands Class";
    }

    /**
     * Returns the tools for frostbite3
     *
     * @return \MAM\Daemon\Engine\Frostbite3\Tools
     */
    public function getTools() {
        return $this->tools;
    }

    /**
     * Sets the maphandle for frostbite3
     */
    public function setMaps() {
        $this->maps = new Maps();
    }

    /**
     * Returns the maphandle for frostbite3
     *
     * @return \MAM\Daemon\Engine\Frostbite3\Maps
     */
    public function getMaps() {
        return $this->maps;
    }

    /**
     * Returns status from an rcon command
     *
     * @param string $data
     * @return bool
     */
    public function getStatus($data) {
        return $this->rcon->getStatus($data);
    }

    /**
     * Executes a rcon command and returns the response.
     *
     * @param string $command command
     * @return mixed
     */
    public function rcon($command) {
        try {
            $command = preg_replace('#(\^.)#', '', $command);
            $out = $this->rcon->rconCommand($command);
            return $out;
        } catch (\Exception $ex) {
            if ($ex->getCode() === 00) {
                $this->logging->error($ex->getMessage());
            }
            $this->logging->notice($ex->getMessage());
            $out = array();
            $out[0] = $ex->getMessage();
            return $out;
        }
    }

    /**
     * Gets a server's variable via rcon.
     *
     * @param string $var
     * @return mixed
     */
    public function rconGetDvar($var) {
        $var = $this->rcon("var" . $var)[1];
        if ($var) {
            return $var;
        }
        return false;
    }

    /**
     * Sets a variable via rcon.
     *
     * @param string $var
     * @param string $value
     */
    public function rconSetDvar($var, $value) {
        $this->rcon("vars.$var $value");
        $this->logging->info("Set Dvar $var to '$value'");
    }

    /**
     * Sets (sets) a Dvar via rcon.
     *
     * @param string $dvar
     * @param string $value
     * @return false
     */
    public function rconSetsDvar($dvar, $value) {
        return false;
    }

    /**
     * Says something via rcon. Use {{br}} or \n for new line.
     *
     * @param string $msg
     * @return void
     */
    public function rconSay($msg) {
        $msg = str_replace("{{br}}", "\n", $msg);
        $msg = wordwrap($msg, 120, "\n", false);
        $this->mod->triggerEvent("rconSay", $msg);
        $split = explode("\n", $msg);
        foreach ($split as $value) {
            //  $value = str_replace(array('"', "'"), '`', $value);
            $this->rcon(array("admin.say", $value, "all"));
        }
    }

    /**
     * Yell something via rcon with duration $duration. Only one Line
     *
     * @param string $msg
     * @param int $duration
     * @return void
     */
    public function rconYell($msg, $duration = 5) {
        $msg = str_replace("{{br}}", " ", $msg);
        $this->mod->triggerEvent("rconYell", $msg);
        $split = explode("\n", $msg);
        foreach ($split as $value) {
            $value = str_replace(array('"', "'"), '`', $value);
            $this->rcon("admin.yell '$value' $duration");
        }
    }

    /**
     * Change map (and gametype) via rcon.
     *
     * @param string $map
     * @param string $gametype
     * @param int $rounds
     * @return boolean
     */
    public function rconChangeMap($map, $gametype, $rounds = 2) {
        if (!$this->rconMap($map, $gametype, $rounds)) {
            return false;
        }
        return true;
    }

    /**
     * Sets gametype via rcon.
     *
     * @param string $gametype
     * @param int $rounds
     * @return boolean
     */
    public function rconSetGametype($gametype, $rounds = 2) {
        return $this->rconMap($this->maps->getMapName($this->maps->getCurrentMapID()), $gametype, $rounds);
    }

    /**
     * Sets map via rcon.
     *
     * @param string $map map
     * @param string $gametype gametype
     * @param int $rounds rounds
     * @return boolean
     */
    public function rconMap($map, $gametype, $rounds = 2) {
        $offset = $this->maps->getNextMapID();
        if ($this->maps->add($map, $gametype, $rounds, $offset) !== false) {
            sleep(3); // wegen lag??
            $return = $this->maps->mapRotate();
            if ($return != false) {
                $this->logging->info("Map was changed to: $map");
                return true;
            }
        }
        return false;
    }

    /**
     * Builds and returns a player array with rcon command "status".
     *
     * @param string $map (call-by-reference) contains the mapname
     * @return array
     */
    public function rconPlayerList(&$map = NULL) {
        $return = array();

        $pb = $this->tools->punkBusterStatus();

        //get Map
        $map = $this->rcon("currentLevel")[1];
        $list = $this->rcon("admin.listPlayers all");
        if ($list[12] == 0) {
            return array();
        }
        $list_count = $list[1] * $list[12] + 12;
        //get playerlist by rcon

        /*
          //      string(2) "OK"
          //    [1]=>
          //    string(2) "10"
          //    [2]=>
          //    string(4) "name"
          //    [3]=>
          //    string(4) "guid"
          //    [4]=>
          //    string(6) "teamId"
          //    [5]=>
          //    string(7) "squadId"
          //    [6]=>
          //    string(5) "kills"
          //    [7]=>
          //    string(6) "deaths"
          //    [8]=>
          //    string(5) "score"
          //    [9]=>
          //    string(4) "rank"
          //    [10]=>
          //    string(4) "ping"
          //    [11]=>
          //    string(4) "type"
          //    [12]=>
          //    string(1) "0"
          //  }
         */
        //var_dump($pb);

        for ($i = 13; $i <= $list_count; $i = $i + 10) {
            if (!isset($list[$i])) {
                break;
            }

            if (array_key_exists($list[$i], $pb)) {
                $ipPort = explode(":", $pb[$list[$i]][2]);
                $return[] = array(
                    "pid" => $pb[$list[$i]][0], //PID in bf3??
                    "score" => $list[$i + 7],
                    "ping" => $list[$i + 9],
                    "guid" => strtolower($list[$i + 1]),
                    "name" => $list[$i],
                    "pbguid" => $pb[$list[$i]][1],
                    "ip" => $ipPort[0],
                    "port" => $ipPort[1]
                );
            } else {
                $return[] = array(
                    "pid" => 99, //player is joining
                    "score" => $list[$i + 7],
                    "ping" => $list[$i + 9],
                    "guid" => strtolower($list[$i + 1]),
                    "name" => $list[$i],
                    "pbguid" => false,
                    "ip" => false,
                    "port" => false
                );
            }
        }
        return $return;
    }

    /**
     * Sets nextmap via rcon.
     *
     * @param string $map map
     * @param string $gametype gametype
     * @param int $rounds rounds
     * @return int
     */
    public function rconSetNextMap($map, $gametype, $rounds = 2) {
        $offset = $this->maps->getNextMapID();
        $return = $this->maps->add($map, $gametype, $rounds, $offset);
        return $return;
    }

    /**
     * Runs a map restart
     *
     * @param boolean $complete If true, a complete map restart is performed. (In BF4 restart whole round
     * @return void
     */
    public function rconMapRestart($complete = false) {

        $this->logging->info("Performing round restart");
        $this->maps->restartRound();
    }

    /**
     * Shutdown the game server.
     *
     * @param int $graceful
     * @param int $delay
     * @return void
     */
    public function rconKillServer($graceful = 1, $delay = 3) {
        $this->rconSay($this->mod->getLngString("killserver"));

        $this->rcon("admin.shutDown $graceful $delay");
        $this->logging->error("Gameserver was shut down");
    }

    /**
     * Load nextmap via rcon mapRotate.
     *
     * @return void
     */
    public function rconMapRotate() {
        $this->maps->mapRotate();
        $this->logging->info("Map was rotated");
    }

    /**
     * Rotates Round
     *
     * @return void
     */
    public function rconRoundRotate() {
        $this->maps->roundRotate();
        $this->logging->info("Round was rotated");
    }

    /**
     * Returns the nextmap.
     *
     * @tutorial 0=>map, 1=>gametype and "map"=>map, "gametype"=>gametype
     * @return array
     */
    public function rconGetNextMap() {
        $index = $this->maps->getNextMapID();
        $map = strtolower($this->maps->getMapName($index));
        $gametype = $this->maps->getShortGamemode($this->maps->getGametype($index));

        $next = array("map" => $map, "gametype" => $gametype, 0 => $map, 1 => $gametype);

        return $next;
    }

    /**
     * Kicks all player from server.
     *
     * @param String $msg Message
     * @return void
     */
    public function rconKickAll($msg = "Kick everybody") {
        foreach ($this->players as $key => $value) {
            $this->rcon("admin.kickPlayer $key \"$msg\"");
        }
        $this->logging->info("Everyone got kicked from the server");
    }

    /**
     * Returns an array with the dvarlist from the server (/No Dvar list on BF4 server)
     *
     * @param string $str
     * @return array
     */
    public function rconDvarList($str = "") {
        return false;
    }

    /**
     * Changes the map to $map with "devmap"!  Not in BF4
     * (The devmap command allows you to perform cheats on your server, so you can get things such as god mode, health, ammo, noclip etc.)
     *
     * @param string $map
     * @return boolean
     */
    public function rconDevMap($map) {
        return false;
    }

    /**
     * Execute file on server. Not in BF4
     *
     * @param string $file
     * @return boolean
     */
    public function rconExec($file) {
        return false;
    }

    /**
     * Executes any rcon command.
     *
     * @param string $command
     * @return string response
     */
    public function rconRcon($command) {
        $this->logging->info("RCON command executed: '$command'");
        return $this->rcon($command);
    }

    /**
     * Tell a player something
     *
     * @param int $pid
     * @param string $msg
     */
    public function rconTell($pid, $msg) {
        $this->players[$this->mod->getPlayerGuidByPID($pid)]->say($msg);
    }

    /**
     * Returns the short Gametype or an error
     *
     * @return string response: gametype|error
     */
    public function rconGetGametype() {
        return $this->rcon("serverInfo")[4];
    }

    /**
     * Switches the player to another team/squad
     *
     * @param \MAM\Daemon\Engine\Frostbite3\Player $player
     * @param int $team
     * @param int $squad
     * @param int $force
     * @return bool
     */
    public function rconSwitchPlayer($player, $team = 0, $squad = 0, $force = 1) {
        $team = ($team == 0) ? $player->getTeam() : $team;
        if ($team != 0) {
            $player->setTeam($team);
        }
        return $this->rcon("admin.movePlayer " . $player->getName() . " $team $squad $force");
    }

    /**
     * Returns an array of teamid => teamname
     *
     * @return array teamid => teamname
     */
    public function rconGetTeamnames() {
        if ($this->rconGetGametype() == "squaddeathmatch0") {
            $teamnames = array(
                1 => "ALFA",
                2 => "BRAVO",
                3 => "CHARLIE",
                4 => "DELTA",
            );
        } else {
            $teamids = $this->rcon("vars.teamFactionOverride");
            $longnames = $this->maps->getTeamnames();
            $teamnames = array(
                1 => $longnames[$teamids[1]],
                2 => $longnames[$teamids[2]],
                3 => $longnames[$teamids[3]],
                4 => $longnames[$teamids[4]]
            );
        }
        return $teamnames;
    }

    /**
     * Returns an array of squadid => squadname
     *
     * @return array squadid => squadname
     */
    public function rconGetSquadnames() {
        return $this->maps->getSquadnames();
    }

    /**
     * Kill a player without any stats effect
     *
     * @param \MAM\Daemon\Engine\Frostbite3\Player $player
     * @return bool
     */
    public function rconKillPlayer($player) {
        return $this->getStatus($this->rcon("admin.killPlayer " . $player->getName()));
    }

    /**
     * Renew the rcon connection
     *
     * @return void
     */
    public function renewRconConnection() {
        $this->logging->info("Renewing rcon connection");
        $this->rcon->renewConnection();
    }
}
