<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Libraries\ErrorHandler;

/**
 * Errorhandling class
 *
 * This class manages the erros, rewrite the messages and prints it
 * It also creates stacktraces for debugging
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class ErrorHandler {
    /**
     * Hide stacktrace after manual kill
     * @var boolean
     */
    private $hide_trace = false;
    /**
     * Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    private $logging;

    /**
     * Constructor
     */
    public function __construct() {
        $registry = \MAM\Daemon\Core\Registry::getInstance();
        $this->logging = $registry->logging;
    }

    /**
     * Returns a string if object is printed
     *
     * Notice: all plugins which where moved to core needs a __tostring function
     * to get it working with the permissions system. Therefore the return of __tostring must me the
     * namespace like deamon, punishment, permissions ...
     *
     * @see http://php.net/manual/en/language.oop5.magic.php#object.tostring
     * @return string
     */
    public function __toString() {
        return 'Error Handler Class';
    }

    /**
     * Checks for a fatal error, work around for set_error_handler not working on fatal errors.
     */
    public function checkForFatal() {
        $error = error_get_last();
        if ($error["type"] == E_ERROR ||
            $error["type"] == E_PARSE ||
            $error["type"] == E_CORE_ERROR ||
            $error["type"] == E_CORE_WARNING ||
            $error["type"] == E_COMPILE_ERROR ||
            $error["type"] == E_COMPILE_WARNING
        ) {
            $this->errorHandler($error["type"], $error["message"], $error["file"], $error["line"]);
        }

        echo "\r\n_____\r\nFatal Error killed us :-(\r\n\r\n";
    }

    /**
     * Catchs Ctrl C and other kills
     *
     * @param int $signal The killing signal
     */
    public function signalHandler($signal) {
        switch ($signal) {
            case SIGTERM:
            case SIGKILL:
            case SIGINT:
            default :
                $this->hide_trace = true;
                $this->logging->error("Got killed by user input (Ctrl + c)");
                break;
        }
    }

    /**
     * Throws a PHP Trace
     *
     * @param Exception $e The Error
     * @return string The trace
     */
    public static function getPHPTrace($e) {
        $count = 0;
        $trace = array();

        $trace[] = "========================";
        $trace[] = "PHP Function Stack:";
        $trace[] = "";

        foreach ($e->getTrace() as $frame) {

            $args = "";
            if (isset($frame['args'])) {
                $args = array();
                foreach ($frame['args'] as $arg) {
                    if (is_string($arg)) {
                        $args[] = "'" . $arg . "'";
                    } elseif (is_array($arg)) {
                        $args[] = 'array';
                    } elseif (is_null($arg)) {
                        $args[] = 'NULL';
                    } elseif (is_bool($arg)) {
                        $args[] = ($arg) ? "true" : "false";
                    } elseif (is_object($arg)) {
                        $args[] = get_class($arg);
                    } elseif (is_resource($arg)) {
                        $args[] = get_resource_type($arg);
                    } else {
                        $args[] = $arg;
                    }
                }
                $args = implode(", ", $args);
            }

            if (!array_key_exists("file", $frame)) {
                $frame["file"] = "unknown";
                $frame["line"] = "?";
            }

            $trace[] = $count . "# " . $frame["file"] . "=>" . $frame["line"] . ": called " . $frame["function"] . "(" . $args . ")";
            $count++;
        }
        $trace[] = "";
        $trace[] = "========================";

        return $trace;
    }

    /**
     * Implements a new error handler
     *
     * @param int $errno
     * @param string $errstr
     * @param string $errfile
     * @param int $errline
     * @return boolean
     */
    public function errorHandler($errno, $errstr, $errfile, $errline) {

        if (error_reporting() == 0 || $this->hide_trace) {
            return;
        }

        $errortype = array(
            E_ERROR => 'Error',
            E_WARNING => 'Warning',
            E_PARSE => 'Parsing Error',
            E_NOTICE => 'Notice',
            E_CORE_ERROR => 'Core Error',
            E_CORE_WARNING => 'Core Warning',
            E_COMPILE_ERROR => 'Compile Error',
            E_COMPILE_WARNING => 'Compile Warning',
            E_USER_ERROR => 'User Error',
            E_USER_WARNING => 'User Warning',
            E_USER_NOTICE => 'User Notice',
            E_STRICT => 'Runtime Notice',
            E_RECOVERABLE_ERROR => 'Catchable Fatal Error'
        );

        $dir = dirname(__FILE__);
        $errfile = substr(str_replace($dir, "", $errfile), 1);

        if ($errno == E_ERROR || $errno == E_CORE_ERROR || $errno == E_COMPILE_ERROR || $errno == E_USER_ERROR) {
            $this->logging->error("$errortype[$errno] in $errfile:$errline => $errstr");
        } elseif ($errno == E_WARNING || $errno == E_USER_WARNING || $errno == E_CORE_WARNING || $errno == E_COMPILE_WARNING) {
            $this->logging->warning("$errortype[$errno] in $errfile:$errline => $errstr");
        } else {
            $this->logging->notice("$errortype[$errno] in $errfile:$errline => $errstr");
        }

        if (DEBUG) {
            $e = new \Exception;
            foreach (self::getPHPTrace($e) as $line) {
                $this->logging->debug($line);
            }
        }

        return true;
    }
}
