<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license http://www.creativecommons.org/licenses/by-nc-nd/4.0/ Creative Commons BY-NC-ND 4.0
 * @version 1.0.0-Beta+7
**/

namespace MAM\Daemon\Libraries\Jobs;

/**
 * Allows to declare jobs that will be executed in the future.
 *
 * @author Jenz <yenz@manuadminmod.de>
 * @author Mirko911 <mirko911@manuadminmod.de>
 */
class Jobs {
    /**
     * The array of jobs to execute
     * @var array
     */
    private $jobs = array();

    /**
     * Call this function to execute all pending jobs.
     */
    public function executeJobs() {
        $job_count = count($this->jobs);
        $microtime = microtime(true);
        for ($i = 0; $i < $job_count; $i++) {
            if (!$this->jobs[$i]['cron']) {
                if ($this->jobs[$i]['time'] < $microtime) {
                    call_user_func_array($this->jobs[$i]['callback'], $this->jobs[$i]['parameters']);
                    unset($this->jobs[$i]);
                }
            } else {
                if ($this->jobs[$i]['next_execute'] < $microtime) {
                    if ($this->jobs[$i]['limit'] != -1) {
                        if ($this->jobs[$i]['executed'] >= $this->jobs[$i]['limit']) {
                            unset($this->jobs[$i]);
                            continue;
                        }
                        $this->jobs[$i]['executed']++;
                    }
                    call_user_func_array($this->jobs[$i]['callback'], $this->jobs[$i]['parameters']);
                    $this->jobs[$i]['next_execute'] = $microtime + $this->jobs[$i]['time'];
                }
            }
        }

        $this->jobs = array_values($this->jobs);
    }

    /**
     * Adds a single job. Will be executed only once.
     * @param int|float $sec time in seconds in which the callback will be executed, float possible (1.5)
     * @param callable $callback a valid callback, see http://www.php.net/manual/en/language.types.callable.php
     * @param array $parameters optional array with parameters for the callback function
     */
    public function addSingleJob($sec, callable $callback, array $parameters = array()) {
        $this->jobs[] = array(
            'cron' => false,
            'time' => microtime(true) + $sec,
            'callback' => $callback,
            'parameters' => $parameters
        );
    }

    /**
     * Adds a periodic job. Will be executed periodically.
     *
     * This functions is an easy way for executing a function every x seconds
     * or minutes. So you don't need to define an extra counter for time events in your function.
     * With this function, it's also possible to execute a job only for a limited count.
     *
     * @param int $time Execute the command all x secs/mins/hours
     * @param string $time_type s/m/h/d define if you want to execute it all x seconds / minutes / hours / days
     * @param int $execute_limit Set a limit for executions. If unlimited use -1
     * @param callable $callback
     * @param array $parameters
     */
    public function addCronJob($time, $time_type, $execute_limit, callable $callback, array $parameters = array()) {
        switch ($time_type) {
            case 'm':
                $next_execute = $time * 60;
                break;
            case 'h':
                $next_execute = $time * 60 * 60;
                break;
            case 'd':
                $next_execute = $time * 60 * 60 * 24;
                break;
            default:
                $next_execute = $time;
                break;
        }

        $this->jobs[] = array(
            'cron' => true,
            'time' => $next_execute,
            'next_execute' => microtime(true) + $next_execute,
            'callback' => $callback,
            'parameters' => $parameters,
            'limit' => $execute_limit,
            'executed' => 0,
        );
    }

    /**
     * Adds a job that will be called every time
     * @param callable $callback
     * @param array $parameters
     */
    public function addEveryTimeJob(callable $callback, array $parameters = array()) {
        $this->addCronJob(0, 's', -1, $callback, $parameters);
    }

    /**
     * Deletes jobs so they will not be executed anymore.
     *
     * The job will be matched by the callback. So if there are multiple jobs with the
     * same callable, they all will be deleted.
     * @param callable $callback
     * @return int count of deleted jobs
     */
    public function deleteJob(callable $callback) {
        $job_count = count($this->jobs);
        $job_delete_count = 0;
        for ($i = 0; $i < $job_count; $i++) {
            if ($this->jobs[$i]['callback'] === $callback) {
                unset($this->jobs[$i]);
                $job_delete_count++;
            }
        }
        $this->jobs = array_values($this->jobs);
        return $job_delete_count;
    }
}
