<?php
/**
 * ManuAdminMod
 *
 * This is a powerful and platform independent software for administrating game servers of various kinds.
 * If you need help with installing or using this software, please visit our website at: www.ManuAdminMod.de
 * If you want to obtain additional permissions extending the license, contact us at: Webmaster@ManuAdminMod.de
 *
 * @copyright www.ManuAdminMod.de
 * @license Creative Commons BY-NC-ND 4.0 <http://www.creativecommons.org/licenses/by-nc-nd/4.0/>
 * @version 1.0.0-Beta+7
 * */

namespace MAM\Daemon\Libraries\Permissions;

use MAM\Daemon\Core\Registry;
use MAM\Daemon\Libraries\Helper\Helper;

/**
 * Permissions Class
 *
 * This class handles the whole permissions system for MAM
 * The idea of permissions is based on the minecraft permissions system.
 *
 * This permissions system supports wildcards, inheritance, and negated permissions
 * pluginnamespace.command would give you access to use !command
 * pluginnamespace.command.admin would give you access to use !command and affect the admin group
 * * gives you a wildcard for everything
 * pluginnamespace.* gives you a wildcard for pluginnamespace
 * -* negated everything
 * -pluginnamespace.* just negated pluginnamespace
 *
 * @author Mirko911 <mirko911@manuadminmod.de>
 * @version 1.0.0
 */
class Permissions {
    /**
     * Class for player related stuff
     *
     * @var \MAM\Daemon\Core\Mod
     */
    protected $mod;
    /**
     * Class for player related stuff
     *
     * @var \MAM\Daemon\Core\Player
     */
    protected $players;
    /**
     * Class for Logging
     *
     * @var \MAM\Daemon\Libraries\Logging\LogHandler
     */
    protected $logging;
    /**
     * Caching for permissions to speed up the function
     * @var array $key => $value Where $key is the groupname and $value is the array of permissions
     */
    protected $cache = array();
    /**
     * The filesize of the groups file.
     * @var int
     */
    private $groups_file_size = 0;

    /**
     * Constructor of the class.
     */
    public function __construct() {
        $registry = Registry::getInstance();
        $this->players = &$registry->players;
        $this->mod = $registry->mod;
        $this->logging = $registry->logging;

        $this->mod->registerEvent('parseConfig', 'rebuildPermissionsCache', $this);
    }

    /**
     * Checks if a player has a permission
     *
     * The Permission System works like the minecraft permissions.
     * If zou check pluginxy.command.ignore and the player has pluginxy.* it will return true
     *
     * @param   string      $guid               Guid of the Player
     * @param   string      $needed_permission  Permission like pluginxyz.foo.*
     * @return  boolean     true if you have the permissions, false if not
     */
    public function hasPermissions($guid, $needed_permission) {
        $group = $this->players[$guid]->getGroup();

        $permissions = array();
        $permissions = $this->getCachedPermissions($group);

        $needed_permission = strtolower($needed_permission);

        $perm = $this->checkPermission(explode('.', $needed_permission), $permissions);
        $negated = $this->checkPermission(explode('.', '-' . $needed_permission), $permissions, true);

        return ($perm && !$negated) ? true : false;
    }

    /**
     * Checks if a group has the permissions to execute a specific oommad
     *
     * @param   string      $group              Name of the group
     * @param   string      $needed_permission  Permission like daemon.cointoss.canexecute
     * @return  boolean     true if you have the permissions, false if not
     */
    public function canExecuteCommand($group, $needed_permission) {
        $permissions = array();
        if (!$group) {
            $this->logging->notice('Group ' . $group . ' doesn\'t exist');
        }

        $permissions = $this->getCachedPermissions($group);

        $perm = $this->checkPermission(explode('.', $needed_permission), $permissions);
        $negated = $this->checkPermission(explode('.', '-' . $needed_permission), $permissions, true);

        return ($perm && !$negated) ? true : false;
    }

    /**
     * Rebuilds the permissions cache
     *
     * The Adminmod builds an multidiminsional array of the permissions
     * To avoid rebuilding of the array every time, we'll cache it.
     * The cache gets a rebuild if the size of the groups file changed
     */
    public function rebuildPermissionsCache() {
        $filesize = filesize($this->mod->getConfigDir() . DIRECTORY_SEPARATOR . 'groups.json');
        if ($filesize != $this->groups_file_size) {
            $this->cache = array();
            foreach (array_keys($this->mod->getGroups()) as $group) {

                $permissions = array();
                $this->cache[$group] = $this->getPermissions($group, $permissions);
            }
            $this->logging->info('Rebuild permissions cache');
            $this->groups_file_size = $filesize;
        }
    }

    /**
     * Returns the permissions from cached array
     *
     * @param   string      $group      Name of the group
     * @return  array       Array with all Permissions. This Array is multidimensional
     */
    private function getCachedPermissions($group) {
        if (array_key_exists($group, $this->cache)) {
            return $this->cache[$group];
        } else {
            $permissions = array();
            $this->cache[$group] = $this->getPermissions($group, $permissions);
            return $this->cache[$group];
        }
    }

    /**
     * Recursive function to build permissions array
     *
     * This functions checks all permissions and inheritance permissions and creates an array of permissions
     * which can be used to check if a player has a permission or not
     *
     * @param   string      $group          The Group where you want to get the perms from
     * @param   array       $permissions    Array of existing permissions
     * @return  array       array with permissions
     */
    private function getPermissions($group, &$permissions) {
        $group = $this->mod->getGroup($group);
        if (array_key_exists('inheritance', $group)) {
            foreach ($group['inheritance'] as $subgroup_name) {
                $subgroup = $this->mod->getGroup($subgroup_name);
                if ($subgroup != false) {
                    $permissions = $this->getPermissions($subgroup_name, $permissions);
                }
            }
        }

        if (!array_key_exists('permissions', $group)) {
            $permissions_neu = array();
        } else {
            $permissions_neu = $group['permissions'];
            $permissions_neu = array_map(
                function($permission) {
                $permission = strtolower($permission);
                $tmpArr = explode('.', $permission);
                return Helper::convertArraytoMultidimensional($tmpArr);
            }, $permissions_neu
            );
        }

        foreach ($permissions_neu as $entry) {
            $permissions = array_merge_recursive($permissions, $entry);
        }
        return $permissions;
    }

    /**
     * Permission Check
     *
     * Checks if a negated permissions blocks the original one
     *
     * @param   string      $needed Permissions you want to check
     * @param   array       $permissions Array of permissions you have
     * @param   boolean     $negated check negated permissions
     * @return  boolean     true if you have the permission. False if not. Notice: if permission is negated, it will also return true
     */
    private function checkPermission($needed, $permissions, $negated = false) {
        $check_wildcard = ($negated) ? '-*' : '*';
        $check = reset($needed);

        if (in_array($check_wildcard, $permissions)) {
            return true;
        }

        if (count($check) === 1 && in_array($check, $permissions)) {
            $permission_tocheck = $permissions[array_search($check, $permissions)];
            if (!is_array($permission_tocheck)) {
                if (next($needed) !== false) {
                    return false;
                } else {
                    return true;
                }
            } else {
                if (in_array('* ', $permission_tocheck)) {
                    return true;
                }
            }
        }
        unset($needed[key($needed)]);
        if (is_string($check) && array_key_exists($check, $permissions)) {
            return $this->checkPermission($needed, $permissions[$check]);
        }
        return false;
    }
}
